#!/usr/local/bin/perl
use Socket;

#==============================================================================
# LinkScan
#==============================================================================
# (c) Copyright 1997-2010 Electronic Software Publishing Corporation (Elsop)
# All Rights Reserved.
# LinkScan(TM) and Elsop(TM) are
# Trademarks of Electronic Software Publishing Corporation
# WWW: http://www.elsop.com/  Email: linkscan@elsop.com
#
# See the file docs/license.txt for terms and conditions of use
#==============================================================================
#
# This is a simple emulation of the UNIX sendmail utility for use with
# LinkScan on systems (e.g. Windows machines) that do not support a
# native implementation. 
#
#==============================================================================

# Fetch SMTP host and my host names from command line

$trace = 0;
while ($ARGV[0] =~ /^-/) {
  if ($ARGV[0] =~ m/^-fromhost/i) {
    shift (@ARGV);
    $fromhost = $ARGV[0];
  }
  elsif ($ARGV[0] =~ m/^-smtphost/i) {
    shift (@ARGV);
    $smtphost = $ARGV[0];
  }
  shift (@ARGV);
}
$smtpport = 25;
if ($smtphost =~ s/:(\d+)$//) {$smtpport = $1; }
unless ($smtphost) {die 'Sendmail.pl: Missing smtphost'; }
unless ($fromhost) {die 'Sendmail.pl: Missing fromhost'; }

# Look for some headers followed by a blank line

$toline = $fromline = $subject = $extraheaders = '';
while (<STDIN>) {
  s/[\r\n]//g;
  if (! $_) {last; }
  elsif (m/^From:\s+(.*)/i) {$fromline = $1; }
  elsif (m/^To:\s+(.*)/i) {$toline = $1; }
  elsif (m/^Subject:\s+(.*)/i) {$subject = $1; }
  else {$extraheaders .= $_."\r\n"; }
}

$fromaddr = $fromline;
$fromaddr =~ s/\s//g;
$fromaddr =~ s/.*<([^>]+)>.*/$1/;
unless ($fromaddr) {die 'Sendmail.pl: Missing From: address'; }
unless ($toline) {die 'Sendmail.pl: Missing To: address'; }

@list = split(/,/, $toline);
foreach $item (@list) {
  $item =~ s/\s//g;
  $item =~ s/.*<([^>]+)>.*/$1/;
}

# Establish a connection with the SMTP server

$addr = inet_aton ($smtphost) or die "Sendmail.pl: Error looking up $smtphost: $!";
$that = sockaddr_in ($smtpport, $addr);
socket (S, AF_INET, SOCK_STREAM, 0) or die "Sendmail.pl: Error creating Socket: $!";
connect (S, $that) or die "Sendmail.pl: Error connecting to $smtphost: $!";
select (S); $| = 1; select (STDOUT);
$msg = '';

$send = "CONN $smtphost $smtpport";
while (<S>) {
  s/[\r\n]//g;
  $msg .= $_."\r\n";
  m/^\s*(\d\d\d)(.)/;
  unless ($2 eq '-') {last; }
}
if ($trace) {print "$send\nCONN: $msg"; }

$send = "HELO $fromhost";
print S "$send\r\n";
while(<S>){
  s/[\r\n]//g;
  if ($trace) {print "$send\nHELO: $_\n"; }
  m/^\s*(\d\d\d)(.)/;
  unless ($1 == 250) {die "Sendmail.pl: $send RESP: $_"; }
  unless ($2 eq '-') {last; }
}

$send = "MAIL FROM: <$fromaddr>";
print S "$send\r\n";
while(<S>){
  s/[\r\n]//g;
  if ($trace) {print "$send\nFROM: $_\n"; }
  m/^\s*(\d\d\d)(.)/;
  unless ($1 == 250) {die "Sendmail.pl: $send RESP: $_"; }
  unless ($2 eq '-') {last; }
}

$err = '';
$doneone = 0;
foreach $item (@list) {
  $send = "RCPT TO: <$item>";
  print S "$send\r\n";
  while(<S>){
    s/[\r\n]//g;
    if ($trace) {print "$send\nRCPT: $_\n"; }
    m/^\s*(\d\d\d)(.)/;
    if ($1 == 250) {$doneone++; } else {$err .= "$send RESP: $_\n"; }
    unless ($2 eq '-') {last; }
  }
}
unless ($doneone) {die "Sendmail.pl: $err"; }

$send = "DATA";
print S "$send\r\n";
while(<S>){
  s/[\r\n]//g;
  if ($trace) {print "$send\nDATA: $_\n"; }
  m/^\s*(\d\d\d)(.)/;
  unless ($2 eq '-') {last; }
}

print S "To: $toline\r\n";
print S "From: $fromline\r\n";
print S "Subject: $subject\r\n";
print S "$extraheaders\r\n";
if ($trace) {print "To: $toline\nFrom: $fromline\nSubject: $subject\n$extraheaders\n";}

# Now send the message body followed by a period

while (<STDIN>) {
  s/[\r\n]//g;
  if ($trace && $_ eq ".") {last; }
  print S "$_\r\n";
}
$send = "ENDM";
print S "\r\n.\r\n";
while(<S>){
  s/[\r\n]//g;
  if ($trace) {print "$send\nENDM: $_\n"; }
  m/^\s*(\d\d\d)(.)/;
  unless ($1 == 250) {die "Sendmail.pl: $send RESP: $_"; }
  unless ($2 eq '-') {last; }
}

# Tidy up

print S "QUIT\r\n";
close (S);
if ($err) {die "Sendmail.pl: $err"; }
exit;

