#!/usr/local/bin/perl5
use Socket;

#==============================================================================
# LinkScan
#==============================================================================
# (c) Copyright 1997-2010 Electronic Software Publishing Corporation (Elsop)
# All Rights Reserved.
# LinkScan(TM) and Elsop(TM) are
# Trademarks of Electronic Software Publishing Corporation
# WWW: http://www.elsop.com/  Email: linkscan@elsop.com
#
# See the file docs/license.txt for terms and conditions of use
#==============================================================================
#
# This hash of tools contains three comma delimited fields
# 1 Absolute pathname to executable
# 2 Type of parameter. h = host; d = domain; u = URL
# 3 Description
#
# Note: httphead is a special (hard coded) case
#
#==============================================================================

$Tools{man} = "/usr/bin/man,h,On-line documentation";
$Tools{whois} = "/usr/bin/whois,d,Lookup a domain record";
$Tools{ping} = "/sbin/ping -c 10,h,Send packets to a host";
$Tools{trace} = "/usr/sbin/traceroute,h,Trace the route to a host";
$Tools{nslookup} = "/usr/sbin/nslookup,h,Name server lookup";
$Tools{host} = "/usr/bin/host -v,h,Name server lookup";
$Tools{dig} = "/usr/bin/dig,h,Name server lookup";
$Tools{httphead} = "httphead,u,Display HTTP header for a URL";

# ipw and dpw are very useful
# http://mjhb.marina-del-rey.ca.us/ipw/
# http://mjhb.marina-del-rey.ca.us/dnw/
# $Tools{ipw} = "/usr/local/bin/ipw,h,IP lookup";
# $Tools{dnw} = "/usr/local/bin/dnw,h,Domain lookup";

$thisscript = $0;
$thisscript =~ s#.*/##;
$query = $ENV{'QUERY_STRING'};
@items = split ("&", $query);
foreach $item (@items)  {
  ($key, $value) = split ("=", $item);
  $value =~ tr/+/ /;
  $value =~ s/%(..)/pack ("C", hex($1))/eg;
  $value =~ s/[\r\n]//;
  $keys{$key} = $value;
}
$tool = $keys{Tool};
$param = $keys{Param};

open (STDERR, ">&STDOUT");
select (STDOUT); $| = 1;
print "Content-type: text/html\n\n";
print "<html><head>\n";
print "<title>LinkScan Tools</title>\n";
print "</head><body>\n";
print qq~<h2 align=center>LinkScan Tools</h2>\n~;
print qq~<p align=center>[<a href="$thisscript?$query&Help=Help">Help</a>]</p>\n~;
print qq~<form action="$thisscript" method="GET">\n<p>\n~;
foreach $item (sort keys %Tools) {
  ($path, $type, $descr) = split (/,/, $Tools{$item});
  print qq~<input type="radio" name="Tool" value="$item"~;
  if ($tool eq $item) {print " checked"; }
  print qq~><a href="$thisscript?Tool=man&Param=$item"><b>$item</b></a> $descr<br>\n~;
}
print qq~</p>\n<p>Enter URL, Host or Domain:<br>\n~;
print qq~<input type="text" name="Param" value="$keys{Param}" size="50">\n~;
print qq~<input type="submit" name="Submit" value="Submit">\n</p></form>\n~;

if ($keys{Help}) {&PrintHelp; }
elsif ($Tools{$tool}) {
  ($path, $type) = split (/,/, $Tools{$tool});
  $param =~ s#\w+:/+##;
  unless ($type eq "u") {
    $param =~ s#/.*$##;
    $param =~ s/[^a-zA-Z0-9\._-]//g;               # keep bad chars away from the shell!!!
  }
  if ($type eq "d") {
    if ($param =~ m/\.(\S+\.\S+)$/) {$param = $1; }
  }
  print "<h2>Result of: '$tool $param'</h2>\n";
  print "<pre>\n";
  if ($tool eq "httphead") {
    $param = "http://".$param;
    $param =~ m#^\w+://([\w-\.]+):?(\d*)($|/(.*))#i;
    $host = $1;
    $port = $2;
    $path = $3;
    unless ($port) {$port = 80; }
    unless ($path) {$path = "/"; }
    $iaddr = inet_aton ($host) || print "Lookup Failed\n";
    $that = sockaddr_in ($port, $iaddr);
    socket (SOCK, AF_INET, SOCK_STREAM, 0) || print "Socket Failed\n";
    connect (SOCK, $that) || print "Connect Failed\n";
    select (SOCK); $| = 1; select (STDOUT);
    print SOCK "HEAD $path HTTP/1.0\n";
    print SOCK "User-Agent: Just Checking\n";
    print SOCK "Host: $host:$port\n";
    print SOCK "Accept: */*\r\n\r\n";
    while (<SOCK>) {
      print $_;
    }
    close (SOCK);
  } else {
    if ($^O !~ m/mswin/i) {
      use Symbol;
      use IPC::Open3;
      $WTR = gensym ();
      $RDR = gensym ();
      $pid = open3 ($WTR, $RDR, "", "$path $param") or print  "Error: Can't execute $tool $!\n";
    } else {
      open (FH, "$path $param |") or print  "Error: Can't execute $tool $!\n";
      $RDR = FH;
    }
    while (<$RDR>) {
      s/((_\cH.)+)/&uline($1)/ge;
      s/(((.\cH)+.)+)/&bold($1)/ge;
      s/&/&amp;/g;
      s/</&lt;/g;
      s/>/&gt;/g;
      s/\cC/<u>/g;
      s/\cD/<\/u>/g;
      s/\cE/<b>/g;
      s/\cF/<\/b>/g;
      print $_;
    }
    close ($RDR);
    close ($WTR);
  }
}
print "</pre>\n<h3>Completed</h3>\n";
print qq~<p><font size ="-1">LinkScan Tools Version 5.0<br>\n~;
print qq~&#169; Copyright 1997-2000 <a href="http://www.elsop.com/">~;
print qq~Electronic Software Publishing Corporation</a>\n~;
print "</font></p></body></html>\n";
exit;

sub PrintHelp {
print qq~
<p>LinkScan Tools is an unsupported CGI script that may be used
to execute one of many command line utilities from a web browser
interface. It also includes a special "built-in" option to display
the HTTP headers for a selected URL.</p>
<p>LinkScan Tools must be installed on your server as a CGI
script and configured with the commands you wish to enable
and the absolute pathnames for those commands on your server.</p>
<p>Another tool we find useful is the
<a href="http://mjhb.mdr.ca.us/cgi-bin/ipw.pl">
ipw program, written by Ron Guilmette</a>.</p>
~;
return;
}
sub uline {
my ($t) = shift;
$t =~ s/.\cH//g;
$t = "\cC$t\cD";
return $t;
}
sub bold {
my ($t) = shift;
$t =~ s/.\cH//g;
$t = "\cE$t\cF";
return $t;
}
